@@ -5,38 +5,26 @@
 
 # Please remember to run "make -C docs html" after update "desc" attributes.
 
+import argparse
 import copy
 import grp
 import inspect
-try:
-    import argparse
-except ImportError:  # python 2.6
-    from . import argparse_compat as argparse
 import os
 import pwd
 import re
+import shlex
 import ssl
 import sys
 import textwrap
-import shlex
 
-from gunicorn import __version__
-from gunicorn import _compat
+from gunicorn import __version__, util
 from gunicorn.errors import ConfigError
 from gunicorn.reloader import reloader_engines
-from gunicorn import six
-from gunicorn import util
 
 KNOWN_SETTINGS = []
 PLATFORM = sys.platform
 
 
-def wrap_method(func):
-    def _wrapped(instance, *args, **kwargs):
-        return func(*args, **kwargs)
-    return _wrapped
-
-
 def make_settings(ignore=None):
     settings = {}
     ignore = ignore or ()
@@ -49,8 +37,8 @@ def make_settings(ignore=None):
 
 
 def auto_int(_, x):
-    if x.startswith('0') and not x.lower().startswith('0x'):
-        # for compatible with octal numbers in python3
+    # for compatible with octal numbers in python3
+    if re.match(r'0(\d)', x, re.IGNORECASE):
         x = x.replace('0', '0o', 1)
     return int(x, 0)
 
@@ -71,7 +59,7 @@ class Config(object):
     def __setattr__(self, name, value):
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
-        super(Config, self).__setattr__(name, value)
+        super().__setattr__(name, value)
 
     def set(self, name, value):
         if name not in self.settings:
@@ -125,18 +113,10 @@ class Config(object):
             worker_class.setup()
         return worker_class
 
-    @property
-    def threads(self):
-        return self.settings['threads'].get()
-
-    @property
-    def workers(self):
-        return self.settings['workers'].get()
-
     @property
     def address(self):
         s = self.settings['bind'].get()
-        return [util.parse_address(_compat.bytes_to_str(bind)) for bind in s]
+        return [util.parse_address(util.bytes_to_str(bind)) for bind in s]
 
     @property
     def uid(self):
@@ -197,7 +177,7 @@ class Config(object):
             return env
 
         for e in raw_env:
-            s = _compat.bytes_to_str(e)
+            s = util.bytes_to_str(e)
             try:
                 k, v = s.split('=', 1)
             except ValueError:
@@ -218,6 +198,10 @@ class Config(object):
 
         return True
 
+    @property
+    def reuse_port(self):
+        return self.settings['reuse_port'].get()
+
     @property
     def paste_global_conf(self):
         raw_global_conf = self.settings['raw_paste_global_conf'].get()
@@ -226,7 +210,7 @@ class Config(object):
 
         global_conf = {}
         for e in raw_global_conf:
-            s = _compat.bytes_to_str(e)
+            s = util.bytes_to_str(e)
             try:
                 k, v = re.split(r'(?<!\\)=', s, 1)
             except ValueError:
@@ -240,13 +224,13 @@ class Config(object):
 
 class SettingMeta(type):
     def __new__(cls, name, bases, attrs):
-        super_new = super(SettingMeta, cls).__new__
+        super_new = super().__new__
         parents = [b for b in bases if isinstance(b, SettingMeta)]
         if not parents:
             return super_new(cls, name, bases, attrs)
 
         attrs["order"] = len(KNOWN_SETTINGS)
-        attrs["validator"] = wrap_method(attrs["validator"])
+        attrs["validator"] = staticmethod(attrs["validator"])
 
         new_class = super_new(cls, name, bases, attrs)
         new_class.fmt_desc(attrs.get("desc", ""))
@@ -315,7 +299,7 @@ class Setting(object):
         return self.value
 
     def set(self, val):
-        if not six.callable(self.validator):
+        if not callable(self.validator):
             raise TypeError('Invalid validator: %s' % self.name)
         self.value = self.validator(val)
 
@@ -333,7 +317,7 @@ def validate_bool(val):
 
     if isinstance(val, bool):
         return val
-    if not isinstance(val, six.string_types):
+    if not isinstance(val, str):
         raise TypeError("Invalid type for casting: %s" % val)
     if val.lower().strip() == "true":
         return True
@@ -350,7 +334,7 @@ def validate_dict(val):
 
 
 def validate_pos_int(val):
-    if not isinstance(val, six.integer_types):
+    if not isinstance(val, int):
         val = int(val, 0)
     else:
         # Booleans are ints!
@@ -360,25 +344,59 @@ def validate_pos_int(val):
     return val
 
 
+def validate_ssl_version(val):
+    ssl_versions = {}
+    for protocol in [p for p in dir(ssl) if p.startswith("PROTOCOL_")]:
+        ssl_versions[protocol[9:]] = getattr(ssl, protocol)
+    if val in ssl_versions:
+        # string matching PROTOCOL_...
+        return ssl_versions[val]
+
+    try:
+        intval = validate_pos_int(val)
+        if intval in ssl_versions.values():
+            # positive int matching a protocol int constant
+            return intval
+    except (ValueError, TypeError):
+        # negative integer or not an integer
+        # drop this in favour of the more descriptive ValueError below
+        pass
+
+    raise ValueError("Invalid ssl_version: %s. Valid options: %s"
+                     % (val, ', '.join(ssl_versions)))
+
+
 def validate_string(val):
     if val is None:
         return None
-    if not isinstance(val, six.string_types):
+    if not isinstance(val, str):
         raise TypeError("Not a string: %s" % val)
     return val.strip()
 
 
+def validate_file_exists(val):
+    if val is None:
+        return None
+    if not os.path.exists(val):
+        raise ValueError("File %s does not exists." % val)
+    return val
+
+
 def validate_list_string(val):
     if not val:
         return []
 
     # legacy syntax
-    if isinstance(val, six.string_types):
+    if isinstance(val, str):
         val = [val]
 
     return [validate_string(v) for v in val]
 
 
+def validate_list_of_existing_files(val):
+    return [validate_file_exists(v) for v in validate_list_string(val)]
+
+
 def validate_string_to_list(val):
     val = validate_string(val)
 
@@ -398,7 +416,7 @@ def validate_class(val):
 
 def validate_callable(arity):
     def _validate_callable(val):
-        if isinstance(val, six.string_types):
+        if isinstance(val, str):
             try:
                 mod_name, obj_name = val.rsplit(".", 1)
             except ValueError:
@@ -412,9 +430,9 @@ def validate_callable(arity):
             except AttributeError:
                 raise TypeError("Can not load '%s' from '%s'"
                     "" % (obj_name, mod_name))
-        if not six.callable(val):
-            raise TypeError("Value is not six.callable: %s" % val)
-        if arity != -1 and arity != len(inspect.getargspec(val)[0]):
+        if not callable(val):
+            raise TypeError("Value is not callable: %s" % val)
+        if arity != -1 and arity != util.get_arity(val):
             raise TypeError("Value must have an arity of: %s" % arity)
         return val
     return _validate_callable
@@ -452,7 +470,7 @@ def validate_group(val):
 def validate_post_request(val):
     val = validate_callable(-1)(val)
 
-    largs = len(inspect.getargspec(val)[0])
+    largs = util.get_arity(val)
     if largs == 4:
         return val
     elif largs == 3:
@@ -539,8 +557,11 @@ class Bind(Setting):
     desc = """\
         The socket to bind.
 
-        A string of the form: ``HOST``, ``HOST:PORT``, ``unix:PATH``. An IP is
-        a valid ``HOST``.
+        A string of the form: ``HOST``, ``HOST:PORT``, ``unix:PATH``,
+        ``fd://FD``. An IP is a valid ``HOST``.
+
+        .. versionchanged:: 20.0
+           Support for ``fd://FD`` got added.
 
         Multiple addresses can be bound. ex.::
 
@@ -603,16 +624,20 @@ class WorkerClass(Setting):
 
         The default class (``sync``) should handle most "normal" types of
         workloads. You'll want to read :doc:`design` for information on when
-        you might want to choose one of the other worker classes.
+        you might want to choose one of the other worker classes. Required
+        libraries may be installed using setuptools' ``extras_require`` feature.
 
         A string referring to one of the following bundled classes:
 
         * ``sync``
-        * ``eventlet`` - Requires eventlet >= 0.9.7
-        * ``gevent``   - Requires gevent >= 0.13
-        * ``tornado``  - Requires tornado >= 0.2
+        * ``eventlet`` - Requires eventlet >= 0.24.1 (or install it via
+          ``pip install gunicorn[eventlet]``)
+        * ``gevent``   - Requires gevent >= 1.4 (or install it via
+          ``pip install gunicorn[gevent]``)
+        * ``tornado``  - Requires tornado >= 0.2 (or install it via
+          ``pip install gunicorn[tornado]``)
         * ``gthread``  - Python 2 requires the futures package to be installed
-        * ``gaiohttp`` - Requires Python 3.4 and aiohttp >= 0.21.5
+          (or install it via ``pip install gunicorn[gthread]``)
 
         Optionally, you can provide your own worker by giving Gunicorn a
         Python path to a subclass of ``gunicorn.workers.base.Worker``.
@@ -640,6 +665,11 @@ class WorkerThreads(Setting):
         If it is not defined, the default is ``1``.
 
         This setting only affects the Gthread worker type.
+
+        .. note::
+           If you try to use the ``sync`` worker type and set the ``threads``
+           setting to more than 1, the ``gthread`` worker type will be used
+           instead.
         """
 
 
@@ -669,7 +699,7 @@ class MaxRequests(Setting):
     desc = """\
         The maximum number of requests a worker will process before restarting.
 
-        Any value greater than zero will limit the number of requests a work
+        Any value greater than zero will limit the number of requests a worker
         will process before automatically restarting. This is a simple method
         to help limit the damage of memory leaks.
 
@@ -743,7 +773,14 @@ class Keepalive(Setting):
     desc = """\
         The number of seconds to wait for requests on a Keep-Alive connection.
 
-        Generally set in the 1-5 seconds range.
+        Generally set in the 1-5 seconds range for servers with direct connection
+        to the client (e.g. when you don't have separate load balancer). When
+        Gunicorn is deployed behind a load balancer, it often makes sense to
+        set this to a higher value.
+
+        .. note::
+           ``sync`` worker does not support persistent connections and will
+           ignore this option.
         """
 
 
@@ -857,6 +894,22 @@ class ReloadEngine(Setting):
         """
 
 
+class ReloadExtraFiles(Setting):
+    name = "reload_extra_files"
+    action = "append"
+    section = "Debugging"
+    cli = ["--reload-extra-file"]
+    meta = "FILES"
+    validator = validate_list_of_existing_files
+    default = []
+    desc = """\
+        Extends :ref:`reload` option to also watch and reload on additional files
+        (e.g., templates, configurations, specifications, etc.).
+
+        .. versionadded:: 19.8
+        """
+
+
 class Spew(Setting):
     name = "spew"
     section = "Debugging"
@@ -923,6 +976,21 @@ class Sendfile(Setting):
         """
 
 
+class ReusePort(Setting):
+    name = "reuse_port"
+    section = "Server Mechanics"
+    cli = ["--reuse-port"]
+    validator = validate_bool
+    action = "store_true"
+    default = False
+
+    desc = """\
+        Set the ``SO_REUSEPORT`` flag on the listening socket.
+
+        .. versionadded:: 19.8
+        """
+
+
 class Chdir(Setting):
     name = "chdir"
     section = "Server Mechanics"
@@ -1147,6 +1215,19 @@ class AccessLog(Setting):
         ``'-'`` means log to stdout.
         """
 
+class DisableRedirectAccessToSyslog(Setting):
+    name = "disable_redirect_access_to_syslog"
+    section = "Logging"
+    cli = ["--disable-redirect-access-to-syslog"]
+    validator = validate_bool
+    action = 'store_true'
+    default = False
+    desc = """\
+    Disable redirect access logs to syslog.
+
+    .. versionadded:: 19.8
+    """
+
 
 class AccessLogFormat(Setting):
     name = "access_log_format"
@@ -1179,10 +1260,15 @@ class AccessLogFormat(Setting):
         D            request time in microseconds
         L            request time in decimal seconds
         p            process ID
-        {Header}i    request header
-        {Header}o    response header
-        {Variable}e  environment variable
+        {header}i    request header
+        {header}o    response header
+        {variable}e  environment variable
         ===========  ===========
+
+        Use lowercase for header and environment variable names, and put
+        ``{...}x`` names inside ``%(...)s``. For example::
+
+            %({x-forwarded-for}i)s
         """
 
 
@@ -1232,7 +1318,7 @@ class CaptureOutput(Setting):
     action = 'store_true'
     default = False
     desc = """\
-        Redirect stdout/stderr to Error log.
+        Redirect stdout/stderr to specified file in :ref:`errorlog`.
 
         .. versionadded:: 19.6
         """
@@ -1270,6 +1356,24 @@ class LogConfig(Setting):
     """
 
 
+class LogConfigDict(Setting):
+    name = "logconfig_dict"
+    section = "Logging"
+    cli = ["--log-config-dict"]
+    validator = validate_dict
+    default = {}
+    desc = """\
+    The log config dictionary to use, using the standard Python
+    logging module's dictionary configuration format. This option
+    takes precedence over the :ref:`logconfig` option, which uses the
+    older file configuration format.
+
+    Format: https://docs.python.org/3/library/logging.config.html#logging.config.dictConfig
+
+    .. versionadded:: 19.8
+    """
+
+
 class SyslogTo(Setting):
     name = "syslog_addr"
     section = "Logging"
@@ -1309,6 +1413,10 @@ class Syslog(Setting):
     default = False
     desc = """\
     Send *Gunicorn* logs to syslog.
+
+    .. versionchanged:: 19.8
+       You can now disable sending access logs by using the
+       :ref:`disable-redirect-access-to-syslog` setting.
     """
 
 
@@ -1370,6 +1478,20 @@ class StatsdHost(Setting):
     .. versionadded:: 19.1
     """
 
+# Datadog Statsd (dogstatsd) tags. https://docs.datadoghq.com/developers/dogstatsd/
+class DogstatsdTags(Setting):
+    name = "dogstatsd_tags"
+    section = "Logging"
+    cli = ["--dogstatsd-tags"]
+    meta = "DOGSTATSD_TAGS"
+    default = ""
+    validator = validate_string
+    desc = """\
+    A comma-delimited list of datadog statsd (dogstatsd) tags to append to statsd metrics.
+
+    .. versionadded:: 20
+    """
+
 class StatsdPrefix(Setting):
     name = "statsd_prefix"
     section = "Logging"
@@ -1450,7 +1572,7 @@ class OnStarting(Setting):
     name = "on_starting"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def on_starting(server):
         pass
@@ -1466,7 +1588,7 @@ class OnReload(Setting):
     name = "on_reload"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def on_reload(server):
         pass
@@ -1482,7 +1604,7 @@ class WhenReady(Setting):
     name = "when_ready"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def when_ready(server):
         pass
@@ -1498,7 +1620,7 @@ class Prefork(Setting):
     name = "pre_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def pre_fork(server, worker):
         pass
@@ -1515,7 +1637,7 @@ class Postfork(Setting):
     name = "post_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def post_fork(server, worker):
         pass
@@ -1532,7 +1654,7 @@ class PostWorkerInit(Setting):
     name = "post_worker_init"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def post_worker_init(worker):
         pass
@@ -1549,7 +1671,7 @@ class WorkerInt(Setting):
     name = "worker_int"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def worker_int(worker):
         pass
@@ -1567,7 +1689,7 @@ class WorkerAbort(Setting):
     name = "worker_abort"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def worker_abort(worker):
         pass
@@ -1587,7 +1709,7 @@ class PreExec(Setting):
     name = "pre_exec"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def pre_exec(server):
         pass
@@ -1603,7 +1725,7 @@ class PreRequest(Setting):
     name = "pre_request"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def pre_request(worker, req):
         worker.log.debug("%s %s" % (req.method, req.path))
@@ -1620,7 +1742,7 @@ class PostRequest(Setting):
     name = "post_request"
     section = "Server Hooks"
     validator = validate_post_request
-    type = six.callable
+    type = callable
 
     def post_request(worker, req, environ, resp):
         pass
@@ -1637,7 +1759,7 @@ class ChildExit(Setting):
     name = "child_exit"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def child_exit(server, worker):
         pass
@@ -1656,7 +1778,7 @@ class WorkerExit(Setting):
     name = "worker_exit"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def worker_exit(server, worker):
         pass
@@ -1673,7 +1795,7 @@ class NumWorkersChanged(Setting):
     name = "nworkers_changed"
     section = "Server Hooks"
     validator = validate_callable(3)
-    type = six.callable
+    type = callable
 
     def nworkers_changed(server, new_value, old_value):
         pass
@@ -1772,14 +1894,32 @@ class SSLVersion(Setting):
     name = "ssl_version"
     section = "SSL"
     cli = ["--ssl-version"]
-    validator = validate_pos_int
+    validator = validate_ssl_version
     default = ssl.PROTOCOL_SSLv23
     desc = """\
-    SSL version to use (see stdlib ssl module's)
+    SSL version to use.
+
+    ============= ============
+    --ssl-version Description
+    ============= ============
+    SSLv3         SSLv3 is not-secure and is strongly discouraged.
+    SSLv23        Alias for TLS. Deprecated in Python 3.6, use TLS.
+    TLS           Negotiate highest possible version between client/server.
+                  Can yield SSL. (Python 3.6+)
+    TLSv1         TLS 1.0
+    TLSv1_1       TLS 1.1 (Python 3.4+)
+    TLSv1_2       TLS 1.2 (Python 3.4+)
+    TLS_SERVER    Auto-negotiate the highest protocol version like TLS,
+                  but only support server-side SSLSocket connections.
+                  (Python 3.6+)
+    ============= ============
 
     .. versionchanged:: 19.7
        The default value has been changed from ``ssl.PROTOCOL_TLSv1`` to
        ``ssl.PROTOCOL_SSLv23``.
+    .. versionchanged:: 20.0
+       This setting now accepts string names based on ``ssl.PROTOCOL_``
+       constants.
     """
 
 class CertReqs(Setting):
@@ -1826,16 +1966,28 @@ class DoHandshakeOnConnect(Setting):
     """
 
 
-if sys.version_info >= (2, 7):
-    class Ciphers(Setting):
-        name = "ciphers"
-        section = "SSL"
-        cli = ["--ciphers"]
-        validator = validate_string
-        default = 'TLSv1'
-        desc = """\
-        Ciphers to use (see stdlib ssl module's)
-        """
+class Ciphers(Setting):
+    name = "ciphers"
+    section = "SSL"
+    cli = ["--ciphers"]
+    validator = validate_string
+    default = None
+    desc = """\
+    SSL Cipher suite to use, in the format of an OpenSSL cipher list.
+
+    By default we use the default cipher list from Python's ``ssl`` module,
+    which contains ciphers considered strong at the time of each Python
+    release.
+
+    As a recommended alternative, the Open Web App Security Project (OWASP)
+    offers `a vetted set of strong cipher strings rated A+ to C-
+    <https://www.owasp.org/index.php/TLS_Cipher_String_Cheat_Sheet>`_.
+    OWASP provides details on user-agent compatibility at each security level.
+
+    See the `OpenSSL Cipher List Format Documentation
+    <https://www.openssl.org/docs/manmaster/man1/ciphers.html#CIPHER-LIST-FORMAT>`_
+    for details on the format of an OpenSSL cipher list.
+    """
 
 
 class PasteGlobalConf(Setting):
