@@ -3,30 +3,30 @@
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-from __future__ import print_function
-
 import email.utils
+import errno
 import fcntl
+import html
+import importlib
+import inspect
 import io
+import logging
 import os
-import pkg_resources
 import pwd
 import random
+import re
 import socket
 import sys
 import textwrap
 import time
 import traceback
-import inspect
-import errno
 import warnings
-import logging
-import re
 
-from gunicorn import _compat
+import pkg_resources
+
 from gunicorn.errors import AppImportError
-from gunicorn.six import text_type
 from gunicorn.workers import SUPPORTED_WORKERS
+import urllib.parse
 
 REDIRECT_TO = getattr(os, 'devnull', '/dev/null')
 
@@ -51,44 +51,7 @@ try:
         setproctitle("gunicorn: %s" % title)
 except ImportError:
     def _setproctitle(title):
-        return
-
-
-try:
-    from importlib import import_module
-except ImportError:
-    def _resolve_name(name, package, level):
-        """Return the absolute name of the module to be imported."""
-        if not hasattr(package, 'rindex'):
-            raise ValueError("'package' not set to a string")
-        dot = len(package)
-        for _ in range(level, 1, -1):
-            try:
-                dot = package.rindex('.', 0, dot)
-            except ValueError:
-                msg = "attempted relative import beyond top-level package"
-                raise ValueError(msg)
-        return "%s.%s" % (package[:dot], name)
-
-    def import_module(name, package=None):
-        """Import a module.
-
-The 'package' argument is required when performing a relative import. It
-specifies the package to use as the anchor point from which to resolve the
-relative import to an absolute import.
-
-"""
-        if name.startswith('.'):
-            if not package:
-                raise TypeError("relative imports require the 'package' argument")
-            level = 0
-            for character in name:
-                if character != '.':
-                    break
-                level += 1
-            name = _resolve_name(name[level:], package, level)
-        __import__(name)
-        return sys.modules[name]
+        pass
 
 
 def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
@@ -132,7 +95,7 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
         klass = components.pop(-1)
 
         try:
-            mod = import_module('.'.join(components))
+            mod = importlib.import_module('.'.join(components))
         except:
             exc = traceback.format_exc()
             msg = "class uri %r invalid or not found: \n\n[%s]"
@@ -140,6 +103,23 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
         return getattr(mod, klass)
 
 
+positionals = (
+    inspect.Parameter.POSITIONAL_ONLY,
+    inspect.Parameter.POSITIONAL_OR_KEYWORD,
+)
+
+
+def get_arity(f):
+    sig = inspect.signature(f)
+    arity = 0
+
+    for param in sig.parameters.values():
+        if param.kind in positionals:
+            arity += 1
+
+    return arity
+
+
 def get_username(uid):
     """ get the username for a user id"""
     return pwd.getpwuid(uid).pw_name
@@ -169,13 +149,12 @@ def set_owner_process(uid, gid, initgroups=False):
 
 
 def chown(path, uid, gid):
-    gid = abs(gid) & 0x7FFFFFFF  # see note above.
     os.chown(path, uid, gid)
 
 
 if sys.platform.startswith("win"):
     def _waitfor(func, pathname, waitall=False):
-        # Peform the operation
+        # Perform the operation
         func(pathname)
         # Now setup the wait loop
         if waitall:
@@ -232,33 +211,35 @@ def is_ipv6(addr):
     return True
 
 
-def parse_address(netloc, default_port=8000):
+def parse_address(netloc, default_port='8000'):
     if re.match(r'unix:(//)?', netloc):
         return re.split(r'unix:(//)?', netloc)[-1]
 
+    if netloc.startswith("fd://"):
+        fd = netloc[5:]
+        try:
+            return int(fd)
+        except ValueError:
+            raise RuntimeError("%r is not a valid file descriptor." % fd) from None
+
     if netloc.startswith("tcp://"):
         netloc = netloc.split("tcp://")[1]
+    host, port = netloc, default_port
 
-    # get host
     if '[' in netloc and ']' in netloc:
-        host = netloc.split(']')[0][1:].lower()
+        host = netloc.split(']')[0][1:]
+        port = (netloc.split(']:') + [default_port])[1]
     elif ':' in netloc:
-        host = netloc.split(':')[0].lower()
+        host, port = (netloc.split(':') + [default_port])[:2]
     elif netloc == "":
-        host = "0.0.0.0"
-    else:
-        host = netloc.lower()
+        host, port = "0.0.0.0", default_port
 
-    #get port
-    netloc = netloc.split(']')[-1]
-    if ":" in netloc:
-        port = netloc.split(':', 1)[1]
-        if not port.isdigit():
-            raise RuntimeError("%r is not a valid port number." % port)
+    try:
         port = int(port)
-    else:
-        port = default_port
-    return (host, port)
+    except ValueError:
+        raise RuntimeError("%r is not a valid port number." % port)
+
+    return host.lower(), port
 
 
 def close_on_exec(fd):
@@ -291,7 +272,7 @@ except ImportError:
 
 
 def write_chunk(sock, data):
-    if isinstance(data, text_type):
+    if isinstance(data, str):
         data = data.encode('utf-8')
     chunk_size = "%X\r\n" % len(data)
     chunk = b"".join([chunk_size.encode('utf-8'), data, b"\r\n"])
@@ -317,7 +298,7 @@ def write_nonblock(sock, data, chunked=False):
 
 
 def write_error(sock, status_int, reason, mesg):
-    html = textwrap.dedent("""\
+    html_error = textwrap.dedent("""\
     <html>
       <head>
         <title>%(reason)s</title>
@@ -327,7 +308,7 @@ def write_error(sock, status_int, reason, mesg):
         %(mesg)s
       </body>
     </html>
-    """) % {"reason": reason, "mesg": _compat.html_escape(mesg)}
+    """) % {"reason": reason, "mesg": html.escape(mesg)}
 
     http = textwrap.dedent("""\
     HTTP/1.1 %s %s\r
@@ -335,32 +316,29 @@ def write_error(sock, status_int, reason, mesg):
     Content-Type: text/html\r
     Content-Length: %d\r
     \r
-    %s""") % (str(status_int), reason, len(html), html)
+    %s""") % (str(status_int), reason, len(html_error), html_error)
     write_nonblock(sock, http.encode('latin1'))
 
 
 def import_app(module):
     parts = module.split(":", 1)
     if len(parts) == 1:
-        module, obj = module, "application"
+        obj = "application"
     else:
         module, obj = parts[0], parts[1]
 
     try:
-        __import__(module)
+        mod = importlib.import_module(module)
     except ImportError:
         if module.endswith(".py") and os.path.exists(module):
             msg = "Failed to find application, did you mean '%s:%s'?"
             raise ImportError(msg % (module.rsplit(".", 1)[0], obj))
-        else:
-            raise
-
-    mod = sys.modules[module]
+        raise
 
     is_debug = logging.root.level == logging.DEBUG
     try:
-        app = eval(obj, vars(mod))
-    except NameError:
+        app = getattr(mod, obj)
+    except AttributeError:
         if is_debug:
             traceback.print_exception(*sys.exc_info())
         raise AppImportError("Failed to find application object %r in %r" % (obj, module))
@@ -501,11 +479,12 @@ def to_bytestring(value, encoding="utf8"):
     """Converts a string argument to a byte string"""
     if isinstance(value, bytes):
         return value
-    if not isinstance(value, text_type):
+    if not isinstance(value, str):
         raise TypeError('%r is not a string' % value)
 
     return value.encode(encoding)
 
+
 def has_fileno(obj):
     if not hasattr(obj, "fileno"):
         return False
@@ -551,7 +530,30 @@ def split_request_uri(uri):
         # relative uri while the RFC says we should consider it as abs_path
         # http://www.w3.org/Protocols/rfc2616/rfc2616-sec5.html#sec5.1.2
         # We use temporary dot prefix to workaround this behaviour
-        parts = _compat.urlsplit("." + uri)
+        parts = urllib.parse.urlsplit("." + uri)
         return parts._replace(path=parts.path[1:])
 
-    return _compat.urlsplit(uri)
+    return urllib.parse.urlsplit(uri)
+
+
+# From six.reraise
+def reraise(tp, value, tb=None):
+    try:
+        if value is None:
+            value = tp()
+        if value.__traceback__ is not tb:
+            raise value.with_traceback(tb)
+        raise value
+    finally:
+        value = None
+        tb = None
+
+
+def bytes_to_str(b):
+    if isinstance(b, str):
+        return b
+    return str(b, 'latin1')
+
+
+def unquote_to_wsgi_str(string):
+    return urllib.parse.unquote_to_bytes(string).decode('latin-1')
