@@ -5,30 +5,31 @@
 
 from functools import partial
 import errno
+import os
 import sys
 
 try:
     import eventlet
 except ImportError:
-    raise RuntimeError("You need eventlet installed to use this worker.")
-
-# validate the eventlet version
-if eventlet.version_info < (0, 9, 7):
-    raise RuntimeError("You need eventlet >= 0.9.7")
-
+    raise RuntimeError("eventlet worker requires eventlet 0.24.1 or higher")
+else:
+    from pkg_resources import parse_version
+    if parse_version(eventlet.__version__) < parse_version('0.24.1'):
+        raise RuntimeError("eventlet worker requires eventlet 0.24.1 or higher")
 
 from eventlet import hubs, greenthread
 from eventlet.greenio import GreenSocket
 from eventlet.hubs import trampoline
+from eventlet.wsgi import ALREADY_HANDLED as EVENTLET_ALREADY_HANDLED
 import greenlet
 
-from gunicorn.http.wsgi import sendfile as o_sendfile
-from gunicorn.workers.async import AsyncWorker
+from gunicorn.workers.base_async import AsyncWorker
+
 
 def _eventlet_sendfile(fdout, fdin, offset, nbytes):
     while True:
         try:
-            return o_sendfile(fdout, fdin, offset, nbytes)
+            return os.sendfile(fdout, fdin, offset, nbytes)
         except OSError as e:
             if e.args[0] == errno.EAGAIN:
                 trampoline(fdout, write=True)
@@ -54,6 +55,7 @@ def _eventlet_serve(sock, handle, concurrency):
             gt.link(_eventlet_stop, server_gt, conn)
             conn, addr, gt = None, None, None
         except eventlet.StopServe:
+            sock.close()
             pool.waitall()
             return
 
@@ -77,22 +79,30 @@ def _eventlet_stop(client, server, conn):
 
 
 def patch_sendfile():
-    from gunicorn.http import wsgi
-
-    if o_sendfile is not None:
-        setattr(wsgi, "sendfile", _eventlet_sendfile)
+    setattr(os, "sendfile", _eventlet_sendfile)
 
 
 class EventletWorker(AsyncWorker):
 
     def patch(self):
         hubs.use_hub()
-        eventlet.monkey_patch(os=False)
+        eventlet.monkey_patch()
         patch_sendfile()
 
+    def is_already_handled(self, respiter):
+        if respiter == EVENTLET_ALREADY_HANDLED:
+            raise StopIteration()
+        return super().is_already_handled(respiter)
+
     def init_process(self):
         self.patch()
-        super(EventletWorker, self).init_process()
+        super().init_process()
+
+    def handle_quit(self, sig, frame):
+        eventlet.spawn(super().handle_quit, sig, frame)
+
+    def handle_usr1(self, sig, frame):
+        eventlet.spawn(super().handle_usr1, sig, frame)
 
     def timeout_ctx(self):
         return eventlet.Timeout(self.cfg.keepalive or None, False)
@@ -102,7 +112,7 @@ class EventletWorker(AsyncWorker):
             client = eventlet.wrap_ssl(client, server_side=True,
                                        **self.cfg.ssl_options)
 
-        super(EventletWorker, self).handle(listener, client, addr)
+        super().handle(listener, client, addr)
 
     def run(self):
         acceptors = []
@@ -118,18 +128,17 @@ class EventletWorker(AsyncWorker):
 
         while self.alive:
             self.notify()
-            try:
-                eventlet.sleep(1.0)
-            except AssertionError:
-                self.alive = False
-                break
+            eventlet.sleep(1.0)
 
         self.notify()
         try:
             with eventlet.Timeout(self.cfg.graceful_timeout) as t:
-                [a.kill(eventlet.StopServe()) for a in acceptors]
-                [a.wait() for a in acceptors]
+                for a in acceptors:
+                    a.kill(eventlet.StopServe())
+                for a in acceptors:
+                    a.wait()
         except eventlet.Timeout as te:
             if te != t:
                 raise
-            [a.kill() for a in acceptors]
+            for a in acceptors:
+                a.kill()
