@@ -10,43 +10,38 @@ from datetime import datetime
 from functools import partial
 import time
 
-_socket = __import__("socket")
-
-# workaround on osx, disable kqueue
-if sys.platform == "darwin":
-    os.environ['EVENT_NOKQUEUE'] = "1"
-
 try:
     import gevent
 except ImportError:
-    raise RuntimeError("You need gevent installed to use this worker.")
+    raise RuntimeError("gevent worker requires gevent 1.4 or higher")
+else:
+    from pkg_resources import parse_version
+    if parse_version(gevent.__version__) < parse_version('1.4'):
+        raise RuntimeError("gevent worker requires gevent 1.4 or higher")
+
 from gevent.pool import Pool
 from gevent.server import StreamServer
-from gevent.socket import wait_write, socket
-from gevent import pywsgi
+from gevent import hub, monkey, socket, pywsgi
 
 import gunicorn
 from gunicorn.http.wsgi import base_environ
-from gunicorn.workers.async import AsyncWorker
-from gunicorn.http.wsgi import sendfile as o_sendfile
+from gunicorn.workers.base_async import AsyncWorker
 
 VERSION = "gevent/%s gunicorn/%s" % (gevent.__version__, gunicorn.__version__)
 
+
 def _gevent_sendfile(fdout, fdin, offset, nbytes):
     while True:
         try:
-            return o_sendfile(fdout, fdin, offset, nbytes)
+            return os.sendfile(fdout, fdin, offset, nbytes)
         except OSError as e:
             if e.args[0] == errno.EAGAIN:
-                wait_write(fdout)
+                socket.wait_write(fdout)
             else:
                 raise
 
 def patch_sendfile():
-    from gunicorn.http import wsgi
-
-    if o_sendfile is not None:
-        setattr(wsgi, "sendfile", _gevent_sendfile)
+    setattr(os, "sendfile", _gevent_sendfile)
 
 
 class GeventWorker(AsyncWorker):
@@ -55,14 +50,7 @@ class GeventWorker(AsyncWorker):
     wsgi_handler = None
 
     def patch(self):
-        from gevent import monkey
-        monkey.noisy = False
-
-        # if the new version is used make sure to patch subprocess
-        if gevent.version_info[0] == 0:
-            monkey.patch_all()
-        else:
-            monkey.patch_all(subprocess=True)
+        monkey.patch_all()
 
         # monkey patch sendfile to make it none blocking
         patch_sendfile()
@@ -70,16 +58,12 @@ class GeventWorker(AsyncWorker):
         # patch sockets
         sockets = []
         for s in self.sockets:
-            if sys.version_info[0] == 3:
-                sockets.append(socket(s.FAMILY, _socket.SOCK_STREAM,
-                    fileno=s.sock.fileno()))
-            else:
-                sockets.append(socket(s.FAMILY, _socket.SOCK_STREAM,
-                    _sock=s))
+            sockets.append(socket.socket(s.FAMILY, socket.SOCK_STREAM,
+                fileno=s.sock.fileno()))
         self.sockets = sockets
 
     def notify(self):
-        super(GeventWorker, self).notify()
+        super().notify()
         if self.ppid != os.getppid():
             self.log.info("Parent changed, shutting down: %s", self)
             sys.exit(0)
@@ -114,20 +98,9 @@ class GeventWorker(AsyncWorker):
             server.start()
             servers.append(server)
 
-        try:
-            while self.alive:
-                self.notify()
-                gevent.sleep(1.0)
-
-        except KeyboardInterrupt:
-            pass
-        except:
-            for server in servers:
-                try:
-                    server.stop()
-                except:
-                    pass
-            raise
+        while self.alive:
+            self.notify()
+            gevent.sleep(1.0)
 
         try:
             # Stop accepting requests
@@ -154,13 +127,20 @@ class GeventWorker(AsyncWorker):
 
             # Force kill all active the handlers
             self.log.warning("Worker graceful timeout (pid:%s)" % self.pid)
-            [server.stop(timeout=1) for server in servers]
+            for server in servers:
+                server.stop(timeout=1)
         except:
             pass
 
-    def handle_request(self, *args):
+    def handle(self, listener, client, addr):
+        # Connected socket timeout defaults to socket.getdefaulttimeout().
+        # This forces to blocking mode.
+        client.setblocking(1)
+        super().handle(listener, client, addr)
+
+    def handle_request(self, listener_name, req, sock, addr):
         try:
-            super(GeventWorker, self).handle_request(*args)
+            super().handle_request(listener_name, req, sock, addr)
         except gevent.GreenletExit:
             pass
         except SystemExit:
@@ -169,36 +149,17 @@ class GeventWorker(AsyncWorker):
     def handle_quit(self, sig, frame):
         # Move this out of the signal handler so we can use
         # blocking calls. See #1126
-        gevent.spawn(super(GeventWorker, self).handle_quit, sig, frame)
+        gevent.spawn(super().handle_quit, sig, frame)
 
-    if gevent.version_info[0] == 0:
+    def handle_usr1(self, sig, frame):
+        # Make the gevent workers handle the usr1 signal
+        # by deferring to a new greenlet. See #1645
+        gevent.spawn(super().handle_usr1, sig, frame)
 
-        def init_process(self):
-            # monkey patch here
-            self.patch()
-
-            # reinit the hub
-            import gevent.core
-            gevent.core.reinit()
-
-            #gevent 0.13 and older doesn't reinitialize dns for us after forking
-            #here's the workaround
-            gevent.core.dns_shutdown(fail_requests=1)
-            gevent.core.dns_init()
-            super(GeventWorker, self).init_process()
-
-    else:
-
-        def init_process(self):
-            # monkey patch here
-            self.patch()
-
-            # reinit the hub
-            from gevent import hub
-            hub.reinit()
-
-            # then initialize the process
-            super(GeventWorker, self).init_process()
+    def init_process(self):
+        self.patch()
+        hub.reinit()
+        super().init_process()
 
 
 class GeventResponse(object):
@@ -222,13 +183,13 @@ class PyWSGIHandler(pywsgi.WSGIHandler):
         resp_headers = getattr(self, 'response_headers', {})
         resp = GeventResponse(self.status, resp_headers, self.response_length)
         if hasattr(self, 'headers'):
-            req_headers = [h.split(":", 1) for h in self.headers.headers]
+            req_headers = self.headers.items()
         else:
             req_headers = []
         self.server.log.access(resp, req_headers, self.environ, response_time)
 
     def get_environ(self):
-        env = super(PyWSGIHandler, self).get_environ()
+        env = super().get_environ()
         env['gunicorn.sock'] = self.socket
         env['RAW_URI'] = self.path
         return env
